@@ -4,19 +4,18 @@
  *  
  *  Copyright (C) 2011 Peng Wu <alexepico@gmail.com>
  *  
- *  This program is free software; you can redistribute it and/or modify
+ *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
- * 
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 
@@ -43,8 +42,6 @@ typedef struct _export_iterator_t export_iterator_t;
 typedef enum _lookup_candidate_type_t{
     BEST_MATCH_CANDIDATE = 1,
     NORMAL_CANDIDATE,
-    DIVIDED_CANDIDATE,
-    RESPLIT_CANDIDATE,
     ZOMBIE_CANDIDATE,
     PREDICTED_CANDIDATE,
     ADDON_CANDIDATE,
@@ -470,7 +467,7 @@ bool pinyin_in_chewing_keyboard(pinyin_instance_t * instance,
 /**
  * pinyin_guess_candidates:
  * @instance: the pinyin instance.
- * @offset: the offset in the pinyin keys.
+ * @offset: the lookup offset.
  * @returns: whether a list of tokens are gotten.
  *
  * Guess the candidates at the offset.
@@ -479,6 +476,7 @@ bool pinyin_in_chewing_keyboard(pinyin_instance_t * instance,
 bool pinyin_guess_candidates(pinyin_instance_t * instance,
                              size_t offset);
 
+#if 0
 /**
  * pinyin_guess_full_pinyin_candidates:
  * @instance: the pinyin instance.
@@ -490,6 +488,7 @@ bool pinyin_guess_candidates(pinyin_instance_t * instance,
  */
 bool pinyin_guess_full_pinyin_candidates(pinyin_instance_t * instance,
                                        size_t offset);
+#endif
 
 /**
  * pinyin_choose_candidate:
@@ -787,22 +786,10 @@ bool pinyin_get_candidate_string(pinyin_instance_t * instance,
                                  lookup_candidate_t * candidate,
                                  const gchar ** utf8_str);
 
-/**
- * pinyin_get_n_pinyin:
- * @instance: the pinyin instance.
- * @num: the number of the pinyins.
- * @returns: whether the get operation is successful.
- *
- * Get the number of the pinyins.
- *
- */
-bool pinyin_get_n_pinyin(pinyin_instance_t * instance,
-                         guint * num);
-
 /**
  * pinyin_get_pinyin_key:
  * @instance: the pinyin instance.
- * @index: the index of the pinyin key.
+ * @offset: the offset of the pinyin key.
  * @key: the retrieved pinyin key.
  * @returns: whether the get operation is successful.
  *
@@ -810,13 +797,13 @@ bool pinyin_get_n_pinyin(pinyin_instance_t * instance,
  *
  */
 bool pinyin_get_pinyin_key(pinyin_instance_t * instance,
-                           guint index,
+                           size_t offset,
                            ChewingKey ** key);
 
 /**
  * pinyin_get_pinyin_key_rest:
  * @instance: the pinyin index.
- * @index: the index of the pinyin key rest.
+ * @offset: the offset of the pinyin key rest.
  * @key_rest: the retrieved pinyin key rest.
  * @returns: whether the get operation is successful.
  *
@@ -824,7 +811,7 @@ bool pinyin_get_pinyin_key(pinyin_instance_t * instance,
  *
  */
 bool pinyin_get_pinyin_key_rest(pinyin_instance_t * instance,
-                                guint index,
+                                size_t offset,
                                 ChewingKeyRest ** key_rest);
 
 /**
@@ -857,19 +844,64 @@ bool pinyin_get_pinyin_key_rest_length(pinyin_instance_t * instance,
                                        guint16 * length);
 
 /**
- * pinyin_get_pinyin_key_rest_offset:
+ * pinyin_get_pinyin_offset:
  * @instance: the pinyin instance.
- * @cursor: the cursor.
- * @offset: the offset in the pinyin array.
+ * @cursor: the user cursor.
+ * @offset: the lookup offset.
  * @returns: whether the get operation is successful.
  *
- * Get the offset in the pinyin key array.
+ * Get the lookup offset from the user cursor.
  *
  */
-bool pinyin_get_pinyin_key_rest_offset(pinyin_instance_t * instance,
-                                       guint16 cursor,
-                                       guint16 * offset);
+bool pinyin_get_pinyin_offset(pinyin_instance_t * instance,
+                              size_t cursor,
+                              size_t * offset);
 
+/**
+ * pinyin_get_left_pinyin_offset:
+ * @instance: the pinyin instance.
+ * @offset: the lookup offset.
+ * @left: the left offset.
+ * @returns: whether the get operation is successful.
+ *
+ * Get the left offset from the lookup offset.
+ *
+ */
+bool pinyin_get_left_pinyin_offset(pinyin_instance_t * instance,
+                                   size_t offset,
+                                   size_t * left);
+
+/**
+ * pinyin_get_right_pinyin_offset:
+ * @instance: the pinyin instance.
+ * @offset: the lookup offset.
+ * @right: the right offset.
+ * @returns: whether the get operation is successful.
+ *
+ * Get the right offset from the lookup offset.
+ *
+ */
+bool pinyin_get_right_pinyin_offset(pinyin_instance_t * instance,
+                                    size_t offset,
+                                    size_t * right);
+
+/**
+ * pinyin_get_character_offset:
+ * @instance: the pinyin instance.
+ * @phrase: the utf8 phrase.
+ * @offset: the lookup offset.
+ * @length: the character offset.
+ * @returns: whether the get operation is successful.
+ *
+ * Get the character offset from the lookup offset.
+ *
+ */
+bool pinyin_get_character_offset(pinyin_instance_t * instance,
+                                 const char * phrase,
+                                 size_t offset,
+                                 size_t * length);
+
+#if 0
 /**
  * pinyin_get_raw_full_pinyin:
  * @instance: the pinyin instance.
@@ -881,6 +913,7 @@ bool pinyin_get_pinyin_key_rest_offset(pinyin_instance_t * instance,
  */
 bool pinyin_get_raw_full_pinyin(pinyin_instance_t * instance,
                                 const gchar ** utf8_str);
+#endif
 
 /**
  * pinyin_get_n_phrase:
@@ -908,7 +941,89 @@ bool pinyin_get_phrase_token(pinyin_instance_t * instance,
                              guint index,
                              phrase_token_t * token);
 
-/* hack here. */
+/**
+ * pinyin_get_full_pinyin_auxiliary_text:
+ * @instance: the pinyin instance.
+ * @cursor: the current cursor.
+ * @aux_text: the auxiliary text.
+ * @returns: whether the get operation is successful.
+ *
+ * Get the auxiliary text for full pinyin.
+ *
+ */
+bool pinyin_get_full_pinyin_auxiliary_text(pinyin_instance_t * instance,
+                                           size_t cursor,
+                                           gchar ** aux_text);
+
+/**
+ * pinyin_get_double_pinyin_auxiliary_text:
+ * @instance: the pinyin instance.
+ * @cursor: the current cursor.
+ * @aux_text: the auxiliary text.
+ * @returns: whether the get operation is successful.
+ *
+ * Get the auxiliary text for double pinyin.
+ *
+ */
+bool pinyin_get_double_pinyin_auxiliary_text(pinyin_instance_t * instance,
+                                             size_t cursor,
+                                             gchar ** aux_text);
+
+/**
+ * pinyin_get_chewing_auxiliary_text:
+ * @instance: the pinyin instance.
+ * @cursor: the current cursor.
+ * @aux_text: the auxiliary text.
+ * @returns: whether the get operation is successful.
+ *
+ * Get the auxiliary text for chewing.
+ *
+ */
+bool pinyin_get_chewing_auxiliary_text(pinyin_instance_t * instance,
+                                       size_t cursor,
+                                       gchar ** aux_text);
+
+/**
+ * pinyin_remember_user_input:
+ * @instance: the pinyin instance.
+ * @phrase: the utf8 phrase.
+ * @count: the count of the phrase, -1 to use the default value.
+ * @returns: whether the phrase is remembered.
+ *
+ * Remember the current phrase and pinyin pair.
+ *
+ */
+bool pinyin_remember_user_input(pinyin_instance_t * instance,
+                                const char * phrase,
+                                gint count);
+
+/**
+ * pinyin_is_user_candidate:
+ * @instance: the pinyin instance.
+ * @candidate: the lookup candidate.
+ * @returns: whether the candidate is user candidate.
+ *
+ * Check whether the candidate is user candidate.
+ *
+ */
+bool pinyin_is_user_candidate(pinyin_instance_t * instance,
+                              lookup_candidate_t * candidate);
+
+/**
+ * pinyin_remove_user_candidate:
+ * @instance: the pinyin instance.
+ * @candidate: the lookup candidate.
+ * @returns: whether the remove operation is successful.
+ *
+ * Remove the user candidate from dictionary.
+ *
+ */
+bool pinyin_remove_user_candidate(pinyin_instance_t * instance,
+                                  lookup_candidate_t * candidate);
+
+
+
+/* for compatibility. */
 typedef ChewingKey PinyinKey;
 typedef ChewingKeyRest PinyinKeyPos;
 
